//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Mainly designed and written by Andr Slabber. No Rights Reserved...  Parts by others...
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  File:         diary.cpp
//  Module:       Diary
//  Author:       Andr Slabber
//
//////////////////////////////////////////////////////////////////////////////////////////////
//
//  Description:  This main file of the application contains the methods of the class
//                SDiaryApp. It also contains the one and only SDiaryApp object, that
//                runs the entire program. 
//
//////////////////////////////////////////////////////////////////////////////////////////////

#include <string>
#include "stdafx.h"
#include "diary.h"
#include "diaryDlg.h"
#include "types.h"
#include "toolkit.h"
#include "diary.h"
#include "tools.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function  ::GetCurrentVersionNumber
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  this global routine returns the application version number as a string
//                because some inbetween versions can have several decimal points. Hard to do 
//                with a float...
//
//  Return Value: char pointer to the current version number (char *)
//
//  Parameters:
//   None
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
std::string GetCurrentVersionNumber(void)   
{ 
   TK_TRACER("::GetCurrentVersionNumber(void)");
   return std::string(VERSION_NUMBER); 
}

std::string GetAvailableVersionNumber(void)
{
	TK_TRACER("::GetAvailableVersionNumber(void)");
	CString theURL = "http://www.moorelife.nl/diaryversion.txt";

	// Only read the file once every execution of the program
	// This makes the download statistics of that file a reasonable
	// measure for the world-wide usage of Diary.....
	static CString theContents = "";
	if (theContents.GetLength() == 0)
	{
	    STools::ReadWebFile(theURL, theContents);
	}

	// Check, did we read a valid version number?
	if (theContents.GetLength() != 5 ||
		STools::isDigit(theContents[0]) == false ||
		theContents[1] != '.' ||
		STools::isDigit(theContents[2]) == false ||
		STools::isDigit(theContents[3]) == false)
	{
		MessageBox(0, "Most recent version number\ncould not be retrieved.",
			       "INFO", MB_OK);

		// force re-reading of the file....
		theContents = "";
		return std::string("none");
	}

	std::string theReturn = theContents;
	return theReturn;
}

int GetCurrentVersionNumberAsDecimal()
{
	return MakeVersionDecimal(GetCurrentVersionNumber());
}

int GetAvailableVersionNumberAsDecimal()
{
	return MakeVersionDecimal(GetAvailableVersionNumber());
}

int MakeVersionDecimal(std::string &theVersion)
{
	char theMajor[2];
	char theMinor[3];

	if (theVersion.size() < 4)
	{
		return 0;
	}

	theMajor[0] = theVersion[0];
	theMajor[1] = '\0';

	if (theVersion[1] != '.')
	{
		return 0;
	}

	theMinor[0] = theVersion[2];
	theMinor[1] = theVersion[3];
	theMinor[2] = '\0';

	return (100 * atoi(theMajor) + atoi(theMinor));
}

BEGIN_MESSAGE_MAP(SDiaryApp, CWinApp)
   //{{AFX_MSG_MAP(SDiaryApp)
      // NOTE - the ClassWizard will add and remove mapping macros here.
      //    DO NOT EDIT what you see in these blocks of generated code!
   //}}AFX_MSG
//   ON_COMMAND(ID_HELP, CWinApp::OnHelp)
// I hope commenting this out will avoid Diary call for help if F1 is pressed...
END_MESSAGE_MAP()


//////////////////////////////////////////////////////////////////////////////////////////////
//  Function  SDiaryApp::SDiaryApp()
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  SDiaryApp constructor, which for now only inits the encryption key
//                all significant initialisation is done in SDiaryApp::InitInstance()
//
//  Return Value: n/a
//
//  Parameters:
//   None
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
SDiaryApp::SDiaryApp()
{
   TK_TRACER("SDiaryApp::SDiaryApp(void)");
   mCryptKey = new char[MAX_STRING + 1];
   strcpy_s(mCryptKey, MAX_STRING - 1, "");
}

// The one and only SDiaryApp object
SDiaryApp theApp;
 
//////////////////////////////////////////////////////////////////////////////////////////////
//  Function  SDiaryApp::InitInstance()
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  SDiaryApp initialization, which basically does some MFC specific stuff, 
//                parses the commandline, and starts the main dialog
//
//  Return Value: see MFC (BOOL)
//
//  Parameters:
//   None
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
BOOL SDiaryApp::InitInstance()
{
   TK_TRACER("SDiaryApp::InitInstance(void)");

   AfxInitRichEdit();
   AfxEnableControlContainer();

   RetCode lReturn = RC_OK;

   // open the toolkits logfile
   lReturn = TK_LOG_OPEN;
   if (lReturn != RC_OK)
   {
      fprintf(stderr, "%s: %s\n", TOOLKIT_LOGNAME, RC2Txt(lReturn));
   }
   else
   {
      Logger::Log(MT_INFO, _FILE_, __LINE__, RC_OK, 
                  "%s open, continue with rest of program", TOOLKIT_LOGNAME);
      try
      {
#ifdef TOOLKIT_SELFTEST
         Logger::selfTest();
         Timer::selfTest();
         Tracer::selfTest();
         Exception::selfTest();
#endif

         ParseCommandLine(mCmdLineInfo);

         TK_LOG0("*****  Creating SDiaryDlg Object");

		 SDiaryDlg *dlg = SDiaryDlg::Instance();
         m_pMainWnd = dlg;
         int nResponse = dlg->DoModal();
		 SDiaryDlg::DeleteInstance();

         TK_LOG0("*****  Destroying SDiaryDlg Object");
      }
      TK_STANDARD_CATCHES;
    
      TK_LOG0("close the toolkits logfile again");
      lReturn = TK_LOG_CLOSE;
      if (lReturn != RC_OK)
      {
         fprintf(stderr, "%s: %s\n", TOOLKIT_LOGNAME, RC2Txt(lReturn));
      }
	  delete [] mCryptKey;
   }
 
   return lReturn;
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function  SDiaryApp::SetCryptKey()
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  SDiaryApp::SetCryptKey can be used to store the encryption key with the 
//                SDiaryApp object. This is done here, since all encryption and decryption 
//                takes place here. 
//
//  Return Value: None (void)
//
//  Parameters:
//   char *pKey      [in]  pointer to the encryption key that will be used henceforth
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
void SDiaryApp::SetCryptKey(char *pKey)
{
   TK_TRACER("SDiaryApp::SetCryptKey(char *pKey)");
   strncpy_s(mCryptKey, MAX_STRING, pKey, MAX_STRING);
   mCryptKey[MAX_STRING] = '\0';
 
   // actually, this should put a NULL character right behind the mCryptKey,
   // but instead it puts one at the end of the buffer. If the key is longer than
   // the buffer, this closes the key for good, but if the key is shorter, it does
   // nothing, so leave it like it is, because ANY change in encryption might 
   // break it for the user, and cause compatibility problems....
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function  SDiaryApp::Encode()
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  SDiaryApp::Encode is used to encode a buffer with the stored keyword. 
//                it is basically the same as SDiaryApp::Decode, and takes a pointer to 
//                the buffer, and an int specifying its length. Optionally, an encryption key
//                can be specified. If not, the default encryption key will be used.
//
//  Return Value: None (void)
//
//  Parameters:
//   char *pBuffer   [i/o] text buffer to be encrypted.
//   int   pLength   [in]  the length of the text that needs to be encrypted
//   char *pCryptKey [in]  pointer to the encryption key that will be used for this action
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
void SDiaryApp::Encode(char *pBuffer, int pLength, char *pCryptKey)
{
   TK_TRACER("SDiaryApp::Encode(char *pBuffer, int pLength, char *pCryptKey)");
   char *lKey = mCryptKey;
   if (pCryptKey != NULL)
   {
      lKey = pCryptKey;
   }
   int lCryptByte = 0;

   TK_LOG0("no key? no encryption...");
   if (strlen(lKey) == 0)
   {
      return;
   }

   TK_LOG0("race through all characters in the buffer...");
   for (int i = 0; i < pLength; i++)
   {
      //TK_LOG0("determine which byte of the cryptkey is up for it");
      lCryptByte++;
      if (lCryptByte == (int)strlen(lKey))
      {
         lCryptByte = 0;
      }
      
      //TK_LOG0("en/decrypt given character with given keybyte");
      pBuffer[i] = pBuffer[i] ^ (char)lKey[lCryptByte];
   }
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function  SDiaryApp::Decode()
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  decode is in this case basically the same operation, with the reverse
//                effect. But to make it clear that we are encoding or decoding, I've 
//                split up the two methods. Optionally, an encryption key can
//                be specified. If not, the default encryption key will be used.
//
//  Return Value: None (void)
//
//  Parameters:
//   char *pBuffer   [i/o] text buffer to be encrypted.
//   int   pLength   [in]  the length of the text that needs to be encrypted
//   char *pCryptKey [in]  pointer to the encryption key that will be used for this action
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
void SDiaryApp::Decode(char *pBuffer, int pLength, char *pCryptKey)
{
   TK_TRACER("SDiaryApp::Decode(char *pBuffer, int pLength, char *pCryptKey)");
   char *lKey = mCryptKey;
   if (pCryptKey != NULL)
   {
      lKey = pCryptKey;
   }
   int lCryptByte = 0;

   TK_LOG0("no key? no encryption...");
   if (strlen(lKey) == 0)
   {
      return;
   }

   TK_LOG0("race through all characters in the buffer...");
   for (int i = 0; i < pLength; i++)
   {
      //TK_LOG0("determine which byte of the cryptkey is up for it");
      lCryptByte++;
      if (lCryptByte == (int)strlen(lKey))
      {
         lCryptByte = 0;
      }

      //TK_LOG0("en/decrypt given character with given keybyte");
      pBuffer[i] = pBuffer[i] ^ (char)lKey[lCryptByte];
   }
}


//////////////////////////////////////////////////////////////////////////////////////////////
//  Function  SDiaryApp::EncryptFile()
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  the EncryptFile and DecryptFile methods are used to keep the images in the 
//                Image Dialog safe...
//
//  Return Value: None (void)
//
//  Parameters:
//   char *pFileName [in]  the name of the .JPG file passed in, output will be .JPX
//   char *pCryptKey [in]  pointer to the encryption key that will be used for this action
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
bool SDiaryApp::EncryptFile(char *pFileName, 
                            char *pCryptKey)
{
   TK_TRACER("SDiaryApp::EncryptFile(char *pFileName, char *pCryptKey)");

   TK_LOG0("JPG file is passed in, so make identical name with JPX extension");
   char *lCryptFileName = new char[MAX_STRING];
   strcpy_s(lCryptFileName, MAX_STRING - 1, pFileName);
   lCryptFileName[strlen(pFileName) - 1] = 'x';

   TK_LOG0("Open JPG file for reading binary");
   FILE *lInFile = fopen(pFileName, "rb");
   if (lInFile != NULL)
   {
      TK_LOG0("Open JPX file for writing binary");
      FILE *lOutFile = fopen(lCryptFileName, "wb");
      if (lOutFile != NULL)
      {
         TK_LOG0("While not EOF on JPG");
         while (feof(lInFile) == false)
         {
            TK_LOG0("Read buffer from JPG");
            char lBuffer[MAX_BUFFER];
            int lRead = fread(lBuffer, 1, MAX_BUFFER, lInFile);

            TK_LOG0("Anything read?");
            if (lRead > 0)
            {
               TK_LOG0("Encode buffer");
               this->Encode(lBuffer, lRead);

               TK_LOG0("Write buffer to JPX");
               if (lRead > (int)(fwrite(lBuffer, 1, lRead, lOutFile)))
               {
                  AfxMessageBox("Encrypted file could not be written");
                  break;
               }
            }
         }
         TK_LOG0("Close JPX file");
         fclose(lOutFile);
      }
      else
      {
         return false;
      }

      TK_LOG0("Close JPG file");
      fclose(lInFile);
   }
   else
   {
      return false;
   }

   TK_LOG0("Delete JPG file");
   _unlink(pFileName);
   delete [] lCryptFileName;
   return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function  SDiaryApp::DecryptFile()
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  the EncryptFile and DecryptFile methods are used to keep the images in the 
//                Image Dialog safe...
//
//  Return Value: None (void)
//
//  Parameters:
//   char *pFileName [in]  the name of the .JPG file to be produced, input will be .JPX
//   char *pCryptKey [in]  pointer to the encryption key that will be used for this action
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
bool SDiaryApp::DecryptFile(char *pFileName, char *pCryptKey)
{
   TK_TRACER("SDiaryApp::DecryptFile(char *pFileName, char *pCryptKey)");

   TK_LOG0("JPG file is passed in, so make identical name with JPX extension");
   char *lCryptFileName = new char[MAX_STRING];
   strcpy_s(lCryptFileName, MAX_STRING - 1, pFileName);
   lCryptFileName[strlen(pFileName) - 1] = 'x';

   TK_LOG0("Open JPX file for reading binary");
   FILE *lInFile = fopen(lCryptFileName, "rb");
   if (lInFile != NULL)
   {
      TK_LOG0("Open JPG file for writing binary");
      FILE *lOutFile = fopen(pFileName, "wb");
      if (lOutFile != NULL)
      {
         TK_LOG0("While not EOF on JPX");
         while (feof(lInFile) == false)
         {
            TK_LOG0("Read buffer from JPX");
            char lBuffer[MAX_BUFFER];
            int lRead = fread(lBuffer, 1, MAX_BUFFER, lInFile);

            TK_LOG0("Anything read?");
            if (lRead > 0)
            {
               TK_LOG0("Decode buffer");
               this->Decode(lBuffer, lRead);

               TK_LOG0("Write buffer to JPG");
               if (lRead > (int)(fwrite(lBuffer, 1, lRead, lOutFile)))
               {
                  AfxMessageBox("Decrypted file could not be written");
                  break;
               }
            }
         }
         TK_LOG0("Close JPG file");
         fclose(lOutFile);
      }
      else
      {
		  delete [] lCryptFileName;
         return false;
      }

      TK_LOG0("Close JPX file");
      fclose(lInFile);
   }
   else
   {
	   delete [] lCryptFileName;
      return false;
   }

   // This does no delete, because we do not want to 
   // have to write the encrypted file over and over again...
   delete [] lCryptFileName;
   return true;
}


//////////////////////////////////////////////////////////////////////////////////////////////
//  Function  SDiaryApp::RecodeFile()
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  used for change of password, to decode and encode a file in one go. 
//
//  Return Value: None (void)
//
//  Parameters:
//   char *pFileName   [in]  the name of the input file to be produced, uses tmp.tmp to replace
//   char *pDecryptKey [in]  pointer to the decryption key that will be used for this action
//   char *pEncryptKey [in]  the encryption key or new password...
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
bool SDiaryApp::RecodeFile(char *pFileName, 
                           char *pDecryptKey,
                           char *pEncryptKey)
{
   TK_TRACER("SDiaryApp::RecodeFile(char *pFileName, char *pDecryptKey, char *pEncryptKey)");

   TK_LOG0("Open file for reading and writing binary");
   FILE *lInFile = fopen(pFileName, "rb");
   if (lInFile != NULL)
   {
      FILE *lOutFile = fopen("tmp.tmp", "wb");
      if (lOutFile != NULL)
      {
         TK_LOG0("While not EOF on JPX");
         while (feof(lInFile) == false)
         {
            TK_LOG0("Read buffer from JPX");
            char lBuffer[MAX_BUFFER];
            int lRead = fread(lBuffer, 1, MAX_BUFFER, lInFile);

            TK_LOG0("Anything read?");
            if (lRead > 0)
            {
               TK_LOG0("Recode buffer");
               this->Decode(lBuffer, lRead, pDecryptKey);
               this->Encode(lBuffer, lRead, pEncryptKey);

               TK_LOG0("Write buffer to TMP");
               if (lRead > (int)(fwrite(lBuffer, 1, lRead, lOutFile)))
               {
                  AfxMessageBox("Recoded file could not be written");
                  break;
               }
            }
         }

         fclose(lOutFile);
      }
      fclose(lInFile);

      _unlink(pFileName);
      rename("tmp.tmp", pFileName);
   }
   return true;
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function  SDiaryApp::~SDiaryApp()
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  destructor, does nothing
//
//  Return Value: n/a
//
//  Parameters:
//    None
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
SDiaryApp::~SDiaryApp(void)
{
   TK_TRACER("SDiaryApp::~SDiaryApp(void)");
}

//////////////////////////////////////////////////////////////////////////////////////////////
//  Function  SDiaryApp::IsProperlyDecrypted()
//////////////////////////////////////////////////////////////////////////////////////////////
//  Description:  indicates if a page has been properly decrypted. Too many non-alphabetic 
//                characters will lead to a false result. 
//
//  Return Value: true if readable, false otherwise (bool)
//
//  Parameters:
//    char *pBuffer   [in]  the string to be inspected for readability
//    int   pLength   [in]  the length of the passed in string
//////////////////////////////////////////////////////////////////////////////////////////////
//	Date		Name                  Comments
//	20080921	Andr Slabber         Created
//////////////////////////////////////////////////////////////////////////////////////////////
bool SDiaryApp::IsProperlyDecrypted(char *pBuffer, int pLength)
{
   TK_TRACER("SDiaryApp::IsProperlyDecrypted(char *pBuffer, int pLength)");
   if (pLength < 5)
   {
      return true;
   }
   int lCharCount = 0;
   for (int i = 0; i < pLength; i++)
   {
      char x = pBuffer[i];
      if ((x >= 'a' && x <= 'z') ||
          (x >= 'A' && x <= 'Z') ||
          (x >= '0' && x <= '9') ||
          x == ' ' || x == '\n'  ||
          x == ',' || x == '\r'  ||
          x == '-' || x == ':'   ||
          x == '@' || x == '.'   ||
          x == ';' || x == '/'   ||
          x == '=' || x == '<'   ||
          x == '\\' || x == '>'  ||
          x == '[' || x == ']'   ||
		  x == '{' || x == '}')
      {
         lCharCount++;
      }
   }
   double lSuccessRate =  ((double)lCharCount / (double)pLength);
   if ((pLength < 100 && lSuccessRate > 0.75) ||
       lSuccessRate > 0.95)
   {
      return true;
   }
   return false;
}

