// This file implements all the static methods of the Logger class. This 
// includes methods to open and close the global logfile, and methods to 
// write to this file. These methods are safeguarded against the logfile
// not being open, and don't write in that case. 

#include "stdafx.h"
#include <tchar.h>       // PLATFORM_SPECIFIC for printf with arg list
#include <stdarg.h>

#include "toolkit.h"

// This is the static variable that Logger uses to maintain the handle to the 
// opened logfile. Any method that wants to write to it, should check first to 
// see if it is not NULL......

FILE *Logger::cvLogFile = NULL;
int   Logger::cvIndent  = 0;

// the open method is supposed to be called at the start of execution
// through the TK_LOG_OPEN macro, so it doesn't get executed if TOOLKIT_LOGFILE
// isn't defined. If so, the logfile remains closed, and no logging at all is done.
// the open() method first checks to see if the file is already open, in which 
// case RC_FILE_ALREADY_OPEN is returned. If opening must still be done, it will 
// return RC_FILE_CANNOT_BE_OPENED if opening was unsuccesful, in which case 
// cvLogFile will remain NULL, and nothing will be logged. 


RetCode Logger::open(void)
{
   RetCode lReturn = RC_OK;

   if (cvLogFile != NULL) 
   {
      lReturn = RC_FILE_ALREADY_OPEN;
   }
   else
   {
      cvLogFile = fopen(TOOLKIT_LOGNAME, "w");

      if (cvLogFile == NULL) 
      {
         lReturn = RC_FILE_CANNOT_BE_OPENED;
      }
   }

   return lReturn;
}

// the close method is supposed to be called at the end of execution
// through the TK_LOG_CLOSE macro, so it doesn't get executed if TOOLKIT_LOGFILE
// isn't defined. The close() method first checks to see if the file is closed, 
// in which case RC_FILE_ALREADY_CLOSED is returned. If closing must still be done, 
// it will return RC_FILE_CANNOT_BE_CLOSED if closing was unsuccesful.
// cvLogFile will be set to NULL in this case. 

RetCode Logger::close(void)
{
   RetCode lReturn = RC_OK;

   if (cvLogFile == NULL)
   {
      lReturn = RC_FILE_ALREADY_CLOSED;
   }
   else if (fclose(cvLogFile) != 0)
   {
      lReturn = RC_FILE_CANNOT_BE_CLOSED;
      cvLogFile = NULL;
   }

   return lReturn;
}

// This method allows the programmer to write to the logfile using
// a similat syntax as printf. The only catch is that he must satisfy
// the fixed requirements of the Logger first: message type to indicate 
// what he'll be logging, file and line to indicate the origin in either
// source or input file, and return code to indicate the nature of the 
// message. After that follows the familiar format string, and the 
// variable arguments of the printf-like part...

void Logger::Log(MsgType  pType, 
                 char    *pFile, 
                 int      pLine, 
                 RetCode  pCode, 
                 char    *pFormat,   // format string for rest of message
                 ...)                // additional parms for rest of message
{
   if (cvLogFile == NULL)
   {
     return; 
   }

   char *lLine = new char[MAX_STRINGBUFFER];

   va_list lList;
   va_start(lList, pFormat);

   char *lTypeText = MT2Txt(pType);
   char *lCodeText;

   if (pFormat != NULL)
   {
      _vstprintf_s(lLine, MAX_STRING, pFormat, lList);
      lCodeText = lLine;
   }
   else
   {
      lCodeText = RC2Txt(pCode);
   }
   
   // set up spacer (to indent if tracing)
   char *lSpacer = new char[MAX_STRINGBUFFER];
   memset(lSpacer, ' ', MAX_STRING);
   int lLength = 3 * cvIndent < MAX_STRING ? 3 * cvIndent : MAX_STRING;
   lSpacer[lLength] = '\0';

   // layout the total line
   fprintf(cvLogFile, 
           "%s %-7.7s %-20.20s %05d %04X: %s%s\n", 
           Timer::getTimeStamp(), 
           lTypeText, 
           pFile, 
           pLine,
           pCode, 
           lSpacer, 
           lCodeText);


   va_end(lList);

   delete [] lLine;
   delete [] lSpacer;
}

// this method logs a single dotted line to the logfile
// MAX_LINESIZE can be set in types.h

void Logger::singleLine(void)
{
   for (int i = 0; i < MAX_LINESIZE; i++)
   {
      fprintf(cvLogFile, "-");
   }
   fprintf(cvLogFile, "\n");
}

// this method logs a double dotted line to the logfile
// MAX_LINESIZE can be set in types.h

void Logger::doubleLine(void)
{
   for (int i = 0; i < MAX_LINESIZE; i++)
   {
      fprintf(cvLogFile, "=");
   }
   fprintf(cvLogFile, "\n");
}

#ifdef TOOLKIT_SELFTEST
// this method performs a limited selfTest for the Logger class. 
// It attempts to write log statements both with the macros and 
// the plain Logger::Log() calls. Visual inspection of the log
// is needed to ascertain success, relative to the source below. 

RetCode Logger::selfTest(void)
{
   RetCode lReturn         = RC_OK;

   // Note the start of the selfTest in the logfile
   Logger::doubleLine();
   Logger::Log(MT_INFO, _FILE_, __LINE__, RC_OK, "Starting selfTest for Logger class");

   // do the various tests
   TK_LOG0("Messages made with TK_LOG macros");
   TK_LOG1("TK_LOG%d has one additional param to fit into the string", 1);
   TK_LOG2("TK_LOG%d has %s additional params to fit into the string", 2, "two");
   TK_LOG3("%s %s %s would be very monotonous indeed...", "keeping", "this", "up");
   TK_LOG4("%d, %x and %X are all representations of %s", 9, 9, 9, "nine");

   TK_LOG9("%04X, %d, %s, %x, %s, %d, %ld, %s and %d are all numbers", 
           0, 1, "two", 3, "four", 5, 6L, "seven", 8);

   TK_PRMINT("Example of an int variable", 169);
   TK_PRMLONG("Example of a long variable", 169169L);
   TK_PRMSTRING("Example of a string variable", "variable");
   TK_PRMHEX("Example of a hex int var", 512);
   TK_PRMLHEX("Example of a hex long var", 65535L);
       
   Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, "Messages made with Logger::Log calls");
   Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, "TK_LOG%d has one additional param to fit into the string", 1);
   Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, "TK_LOG%d has %s additional params to fit into the string", 2, "two");
   Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, "%s %s %s would be very monotonous indeed...", "keeping", "this", "up");
   Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, "%d, %x and %X are all representations of %s", 9, 9, 9, "nine");

   Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, 
               "%04X, %d, %s, %x, %s, %d, %ld, %s and %d are all numbers", 
               0, 1, "two", 3, "four", 5, 6L, "seven", 8);
       
   // Note the end of the selfTest in the logfile
   Logger::Log(MT_INFO, _FILE_, __LINE__, RC_OK, "End of selfTest for Logger class"); 
   Logger::doubleLine();

   return lReturn;
}
#endif
