#ifndef TOOLKIT_H_INCLUDED
#define TOOLKIT_H_INCLUDED

// This is the main include file for the toolkit. Any source that 
// wants to use the toolkit will only have to include this file.
// Anything else used for the toolkit will be included from here. 

#include <string.h>

#include "settings.h"
#include "types.h"
#include "logger.h"
#include "timer.h"
#include "tracer.h"
#include "exception.h"

// define _FILE_ to be __FILE__ with the path stripped off. 

#define _FILE_     (char *)(strrchr(__FILE__, '\\') + 1)

// The next macro will alert the user that some work needs to be done at a specific 
// place in the program. It displays a messagebox (always) if that part of the code 
// is hit....

#define TODO(a)  {                                                       \
                    char *lMsg = new char[MAX_STRING];                   \
                    sprintf_s(lMsg, MAX_STRING - 1,                      \
					"File: %s\nLine: %d\nTODO: %s",                      \
                    _FILE_, __LINE__, a);                                \
                    AfxMessageBox(lMsg);                                 \
					delete [] lMsg;                                      \
                 }

// the next ifdef makes sure that the TK_LOG_OPEN and TK_LOG_CLOSE commmands
// have the required effect: opening and closing of the logfile if TOOLKIT_LOGFILE
// is defined, and no effect whatsoever if it isn't....

#ifdef TOOLKIT_LOGFILE

#define TK_LOG_OPEN    Logger::open()     // perform real open
#define TK_LOG_CLOSE   Logger::close()    // perform real close

#else

#define TK_LOG_OPEN    RC_OK   // fake successful open
#define TK_LOG_CLOSE   RC_OK   // fake successful close

#endif

// These macros allow writing to the logfile in a removable way: they call 
// Logger::Log() if TOOLKIT_LOGGING is defined, and nothing otherwise. Sometimes, 
// depending on the parameters, these calls will introduce compiler errors about
// parameters not being able to be fitted correctly. A direct call to Logger::Log()
// using the same parameters can then be used as a replacement, or to debug the
// statement first. 

#ifdef TOOLKIT_LOGGING

#define TK_LOG0(str)                            Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, str);
#define TK_LOG1(str, a)                         Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, str, a);
#define TK_LOG2(str, a, b)                      Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, str, a, b);
#define TK_LOG3(str, a, b, c)                   Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, str, a, b, c);
#define TK_LOG4(str, a, b, c, d)                Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, str, a, b, c, d);
#define TK_LOG5(str, a, b, c, d, e)             Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, str, a, b, c, d, e);
#define TK_LOG6(str, a, b, c, d, e, f)          Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, str, a, b, c, d, e, f);
#define TK_LOG7(str, a, b, c, d, e, f, g)       Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, str, a, b, c, d, e, f, g);
#define TK_LOG8(str, a, b, c, d, e, f, g, h)    Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, str, a, b, c, d, e, f, g, h);
#define TK_LOG9(str, a, b, c, d, e, f, g, h, i) Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, str, a, b, c, d, e, f, g, h, i);

#else

#define TK_LOG0(str)                      
#define TK_LOG1(str, a)                 
#define TK_LOG2(str, a, b)              
#define TK_LOG3(str, a, b, c)           
#define TK_LOG4(str, a, b, c, d)        
#define TK_LOG5(str, a, b, c, d, e)     
#define TK_LOG6(str, a, b, c, d, e, f)          
#define TK_LOG7(str, a, b, c, d, e, f, g)       
#define TK_LOG8(str, a, b, c, d, e, f, g, h)    
#define TK_LOG9(str, a, b, c, d, e, f, g, h, i)

#endif

#ifdef TOOLKIT_PARMS

#define TK_PRMSTRING(name, value) Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, "%s:\t%s", name, value);
#define TK_PRMINT(name, value)    Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, "%s:\t%d", name, value);
#define TK_PRMLONG(name, value)   Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, "%s:\t%ld", name, value);
#define TK_PRMHEX(name, value)    Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, "%s:\t%04X", name, value);
#define TK_PRMLHEX(name, value)   Logger::Log(MT_LOGGING, _FILE_, __LINE__, RC_OK, "%s:\t%08X", name, value);

#else

#define TK_PRMSTRING(name, value) 
#define TK_PRMLONG(name, value)   
#define TK_PRMINT(name, value)    
#define TK_PRMHEX(name, value)    
#define TK_PRMLHEX(name, value)    

#endif

// This macro allows for erasable Tracers, that trace entering and exiting of a method.
// You call it by passing in the name of the Tracer, and it immediately logs its creation
// as the entry of the trace. If the tracer goes out of scope, its destructor prints this
// event to the system logfile. If TOOLKIT_TRACING is not defined, nothing happens. 

#ifdef TOOLKIT_TRACING

#define TK_TRACER(method)      Tracer lTracer(method); 

#else

#define TK_TRACER(method)

#endif

// These macros are active when TOOLKIT_TIMING is defined, and entirely inactive if it 
// isn't. The allow Timers to be defined, and repeatedly started and stopped. If a Timer
// goes out of scope while it is running, it is stopped during its destruction. Starting 
// and stopping of Timers is written to the log, along with the name selected. This feature
// is engaged automatically for any Tracer if TOOLKIT_TIMING is defined. 

#ifdef TOOLKIT_TIMING

#define TK_TIMER_DEF(a, b)      Timer a(b);
#define TK_TIMER_START(a)       a.startTimer();
#define TK_TIMER_STOP(a)        a.stopTimer();

#else

#define TK_TIMER_DEF(a, b) 
#define TK_TIMER_START(a)     
#define TK_TIMER_STOP(a)     

#endif

// if TOOLKIT_THROWS is defined, then simple macros can be invoked to throw a Toolkit 
// Exception, either unconditionally, or only if the parameter passed in does not equal 
// RC_OK. These macros become inactive if TOOLKIT_THROWS is undefined. 

#ifdef TOOLKIT_THROWS

#define TK_THROW(a)      { throw new Exception(_FILE_, __LINE__, a);                   }
#define TK_THROWONRC(a)  { if (a != RC_OK) { throw new Exception(_FILE_, __LINE__, a); }             };
        
#else

#define TK_THROW(a)         
#define TK_THROWONRC(a)  
        
#endif

// If TOOLKIT_ASSERT is defined, TK_ASSERT will throw an Exception if the expression 
// passed in fails to evaluate to true. TK_TEST does the same, but it is always active, 
// even if TOOLKIT_ASSERT is not defined. 

#ifdef  TOOLKIT_ASSERT

#define TK_ASSERT(f) { if (!(f)) { throw new Exception(_FILE_, __LINE__, RC_ASSERT); } }

#else

#define TK_ASSERT(f)

#endif

#define TK_TEST(f) { if (!(f)) { throw new Exception(_FILE_, __LINE__, RC_TESTFAILED); } }

// The macros below define standard catch phrases for the try/catch statement. Depending
// on the definition of TOOLKIT_CATCHES, one of two sets of catches is activated where the 
// macro TK_STANDARD_CATCHES is inserted. 
                                                                                                              
#ifdef TOOLKIT_CATCHES
                                                                                                               
#define TK_STANDARD_CATCHES                                                                                     \
        catch (char *s)                                                                                   \
        {                                                                                                 \
           fprintf(stderr, "Exception(char *) thrown, check the log.\n");                                 \
           Logger::Log(MT_FATAL, _FILE_, __LINE__, RC_FATAL, "Exception(char *) thrown: [%s]", s);        \
        }                                                                                                   \
        catch (Exception *e)                                                                              \
        {                                                                                                 \
           fprintf(stderr, "Toolkit Exception thrown, check the log.\n");                                 \
           e->Log();                                                                                       \
        }                                                                                                 \
        catch (...)                                                                                       \
        {                                                                                                 \
           fprintf(stderr, "An unknown system exception has occurred.\n");                                  \
           Logger::Log(MT_FATAL, _FILE_, __LINE__, RC_FATAL, "An unknown system exception has occurred"); \
        }                                                                                        
                                                                                                   
#else

#define TK_STANDARD_CATCHES                                                                                  \
        catch (char *s)                                                                                   \
        {                                                                                                 \
           fprintf(stderr, "Fatal System Error, call the Developers!\n");                                 \
           Logger::Log(MT_FATAL, _FILE_, __LINE__, RC_FATAL, "Exception(char *) thrown: [%s]", s);        \
        }                                                                                                 \
        catch (Exception *e)                                                                              \
        {                                                                                                 \
           fprintf(stderr, "Fatal System Error, call the Developers!\n");                                  \
           e->Log();                                                                                      \
        }                                                                                                 \
        catch (...)                                                                                       \
        {                                                                                                    \
           fprintf(stderr, "Fatal System Error, call the Developers!\n");                                 \
           Logger::Log(MT_FATAL, _FILE_, __LINE__, RC_FATAL, "An unknown system exception has occurred"); \
        }                                                                                              

#endif                              




#endif   // TOOLKIT_H_INCLUDED